//Autor: Radi Harald

using System;
using System.Collections;

namespace IBMKeyManagement
{
	public class CryptoFacilityHost
	{
		private byte[] kmh0;
		private byte[] kmh1;

		private static Hashtable keyTable = new Hashtable();

		private static CipherAlgorithm cipher = Host.algorithm;
		private static Random rnd = new Random();

		public CryptoFacilityHost()
		{
		}

		#region AdminClient Functions
		/// <summary>
		/// Used to set the host master key (kmh0).
		/// This function is not to be called during normal operation, it is only
		/// used in conjunction with the AdminClient to represent an administrative 
		/// interface to the cryptographic facility.
		/// </summary>
		/// <param name="key">new master key</param>
		public void SetMasterKey(byte[] key)
		{
			kmh0 = key;

			kmh1 = new byte[key.Length];
			kmh0.CopyTo(kmh1, 0);
			kmh1[0] ^= (byte) 1;		/* kmh1 may be a derivate of kmh0 as there
										 * will never be the same data encrypted
										 * with kmh0 and kmh1
										 */
		}

		// only for adminstrative purpose

		/// <summary>
		/// Used to set the host terminal keys.
		/// This function is not to be called during normal operation, it is only
		/// used in conjunction with the AdminClient to represent an administrative 
		/// interface to the cryptographic facility.
		/// </summary>
		/// <param name="terminal">associated terminal</param>
		/// <param name="key">terminal master key</param>
		public void SetTerminalKey(long terminal, byte[] key) 
		{
			keyTable[terminal] = cipher.Encipher(key, kmh1);
		}
		#endregion

		/// <summary>
		///	Enciphers the master session key for an unique terminal
		///	with its own terminal master key (kmt)
		/// </summary>
		/// <param name="host">target host</param>
		/// <param name="rn">random number to derive session key from</param>
		/// <returns></returns>
		public byte[] RecipherSessionKey(long host, byte[] rn)
		{
			if (kmh0 == null) {
				throw new NoMasterKeyPresentException();
			}
			return cipher.Encipher(	cipher.Decipher(kmh1, (byte[]) keyTable[host]),
									cipher.Decipher(kmh0, rn));
		}

		/// <summary>
		/// Creates the session key for a single data transfer only the random 
		/// number (rn) is stored, the actual session key (ks) is derived later 
		/// by via Decipher(kmh0, rn)
		/// </summary>
		/// <returns>random number</returns>
		public byte[] GenerateSessionKey()
		{
			byte[] rn = new byte[cipher.GetKeyLength()];
			rnd.NextBytes(rn);

			return rn;
		}
	}
}
