//Autor: Kumar Michael 0055932/880

using System;
using System.Text;
using System.Net;
using System.Net.Sockets;
using System.Threading;

namespace IBMKeyManagement
{
	public class Host
	{
		//global constants
		public const int HOSTPORT      = 12345; //host listens at this port
		public const int CTLPORT       = 12346; //terminal listens at this port for keys
		public const int RCVPORT       = 12347; //terminal listens at this port for data
		public const int ADMINPORTHOST = 12348; //host listens at this port for administrative connections
		public const int ADMINPORTTERM = 12349; //terminal listens at this port for administrative connections
		
		// algorithm can be public
		public static CipherAlgorithm algorithm = new CipherXor();

		// crypto facility has to be private and secured
		private static CryptoFacilityHost facility = new CryptoFacilityHost();

		private Thread t = null;
		private Thread t2 = null;
		private TcpListener tcpl;
		private TcpListener adm;

		public Host()
		{
			tcpl = new TcpListener(HOSTPORT);
			tcpl.Start();

			adm = new TcpListener(ADMINPORTHOST);
			adm.Start();

			WaitForMasterKey();

			t = new Thread(new ThreadStart(this.HandleConnection));
			t.Name = "HandleConnection";
			t.Start();

			t2 = new Thread(new ThreadStart(this.HandleAdminConnection));
			t2.Name = "HandleAdminConnection";
			t2.Start();
		}

		public void Dispose() {
			t.Abort();
			t2.Abort();
			tcpl.Stop();
			adm.Stop();
		}

		~Host() {
			Dispose();
		}

		public static byte[] HostToByteIp(String host) {
			IPHostEntry termip = Dns.Resolve(host);
			long ip = termip.AddressList[0].Address;
			byte[] rawip = new byte[4];

			rawip[0] = (byte) (ip >> 24);
			rawip[1] = (byte) (ip >> 16 & 255);
			rawip[2] = (byte) (ip >> 8 & 255);
			rawip[3] = (byte) (ip & 255);

			return rawip;
		}

		public static long ByteIpToLongIp(byte[] ip) {
			long dest;

			dest = ip[0] << 24;
			dest += ip[1] << 16;
			dest += ip[2] << 8;
			dest += ip[3];

			return dest;
		}

		#region administrative interface
		/// <summary>
		/// Used to inject the host master key (kmh0).
		/// This function is not to be called during normal operation, it is only
		/// used in conjunction with the AdminClient to represent an administrative 
		/// interface to the cryptographic facility.
		/// </summary>
		protected void WaitForMasterKey() 
		{
			byte[] key = new byte[Host.algorithm.GetKeyLength()];

			Socket admin = adm.AcceptSocket();
			admin.Receive(key, key.Length, SocketFlags.None);
			facility.SetMasterKey(key);
			admin.Close();
		}

		/// <summary>
		/// Used to inject the host terminal keys
		/// This function is not to be called during normal operation, it is only
		/// used in conjunction with the AdminClient to represent an administrative 
		/// interface to the cryptographic facility.
		/// </summary>
		protected void HandleAdminConnection() 
		{
			while (true) {
				byte[] key = new byte[Host.algorithm.GetKeyLength()];
				byte[] host = new byte[4];

				Socket admin = adm.AcceptSocket();
				admin.Receive(key, key.Length, SocketFlags.None);
				admin.Receive(host, host.Length, SocketFlags.None);
				admin.Close();

				facility.SetTerminalKey(ByteIpToLongIp(host), key);
				Console.Out.WriteLine("terminal master key set");
			}
		}
		#endregion
        
		/// <summary>
		/// Handles connections by terminals.
		/// The connecting terminal sends the ip of the peer it wants to talk with,
		/// the host then creates a session key and sends it to both peers who can then
		/// exchange data independently.
		/// </summary>
		protected void HandleConnection() 
		{
			while (true) {
				byte[] ip = new byte[4];	// destination ip
				byte[] keya, keyb;			// keys
							
				Socket terminalA = tcpl.AcceptSocket();
				terminalA.Receive(ip, 4, SocketFlags.None);

				long ipa = ((IPEndPoint) terminalA.RemoteEndPoint).Address.Address;
				long ipb = Host.ByteIpToLongIp(ip);
							
				TcpClient terminalB = new TcpClient();
				terminalB.Connect(new IPAddress(ipb), CTLPORT);

				// generate master session key
				byte[] ks = facility.GenerateSessionKey();

				//genereate enciphered session keys for each terminal
				keya = facility.RecipherSessionKey(ipa, ks);
				keyb = facility.RecipherSessionKey(ipb, ks);

				//send session key to terminal A
				terminalA.Send(keya);
				terminalA.Close();

				//send session key to terminal B
				NetworkStream strm = terminalB.GetStream();
				strm.Write(keyb, 0, keyb.Length);
				strm.Close();

				terminalB.Close();
			}
		}
	}
}
