//Autor: Radi Harald

using System;

namespace IBMKeyManagement
{
	public class CryptoFacilityTerminal
	{
		private byte[] kmt;
		private byte[] ks;

		private static CipherAlgorithm cipher = Host.algorithm;

		public CryptoFacilityTerminal()
		{
		}

		#region AdminClient Functions
		/// <summary>
		/// Used to set the terminal master key (kmt).
		/// This function is not to be called during normal operation, it is only
		/// used in conjunction with the AdminClient to represent an administrative 
		/// interface to the cryptographic facility.
		/// </summary>
		/// <param name="key">new master key</param>
		public void SetMasterKey(byte[] key)
		{
			kmt = key;
		}
		
		#endregion

		/// <summary>
		/// Sets up the crypto facility with the current session key
		/// by deciphering it with the own terminal master key
		/// </summary>
		/// <param name="ksEnc"></param>
		public void SetSessionKey(byte[] ksEnc)
		{
			if (kmt == null) {
				throw new NoMasterKeyPresentException();
			}
			ks = cipher.Decipher(kmt, ksEnc);
		}
		
		/// <summary>
		/// Enciphers data with the currently set session key
		/// </summary>
		/// <param name="data">Plaintext</param>
		/// <returns>Ciphertext</returns>
		public byte[] EncipherWithKS(byte[] data)
		{
			return cipher.Encipher(ks, data);
		}

		/// <summary>
		/// Deciphers data with the currently set session key
		/// </summary>
		/// <param name="data">Ciphertext</param>
		/// <returns>Plaintext</returns>
		public byte[] DecipherWithKS(byte[] data)
		{
			return cipher.Decipher(ks, data);
		}

		public bool IsMasterKeySet()
		{
			return (kmt != null);
		}
	}
}