//Autor: Kumar Michael 0055932/880

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading;
using System.Windows.Forms;

namespace IBMKeyManagement
{
	public class Terminal : System.Windows.Forms.Form
	{
		private System.Windows.Forms.Button ConnectButton;
		private System.Windows.Forms.TextBox HostTextBox;
		private System.ComponentModel.Container components = null;
		private System.Windows.Forms.TextBox TerminalTextBox;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.Label label2;
		private System.Windows.Forms.TextBox DataTextBox;
		private System.Windows.Forms.TextBox MessageTextBox;
		private TcpListener ctrl;
		private TcpListener rcv;
		private System.Windows.Forms.Button ClearButton;
		private Thread t;
		private System.Windows.Forms.Label MasterKeySet;

		private CryptoFacilityTerminal facility;

		private bool sessionKeySet = false;

		public Terminal()
		{
			InitializeComponent();

			ctrl = new TcpListener(Host.CTLPORT);
			rcv = new TcpListener(Host.RCVPORT);

			ctrl.Start();
			rcv.Start();

			t = new Thread(new ThreadStart(this.HandleConnection));
			t.Name = "HandleConnection";
			t.Start();

			facility = new CryptoFacilityTerminal();
		}

		protected void HandleConnection() {
			WaitForTerminalKey();

			while (true) {
				byte[] key = new byte[Host.algorithm.GetKeyLength()];
				byte[] buffer = new Byte[key.Length * 64];	/* buffer has to be a
															 * multiple of keylength
															 */
				Socket control = ctrl.AcceptSocket();
				control.Receive(key, key.Length, SocketFlags.None);

				facility.SetSessionKey(key); //set up the session key
				sessionKeySet = true;

				control.Close();

				TcpClient data = rcv.AcceptTcpClient();
				NetworkStream strm = data.GetStream();
				do {
					int read = strm.Read(buffer, 0, buffer.Length);
					buffer = facility.DecipherWithKS(buffer);
					DataTextBox.Text += Encoding.ASCII.GetString(buffer, 0, read);
				} while (strm.DataAvailable);
				DataTextBox.Text += "\r\n";
				strm.Close();
			}
		}

		/// <summary>
		/// Used to inject the terminal master key (kmt)
		/// This function is not to be called during normal operation, it is only
		/// used in conjunction with the AdmminClient to represent an administrative 
		/// interface to the cryptographic facility.
		/// </summary>
		protected void WaitForTerminalKey() 
		{
			byte[] key = new byte[Host.algorithm.GetKeyLength()];
			TcpListener adm = new TcpListener(Host.ADMINPORTTERM);

			adm.Start();

			Socket admin = adm.AcceptSocket();
			admin.Receive(key, key.Length, SocketFlags.None);
			admin.Close();

			facility.SetMasterKey(key);

			MasterKeySet.Text = "Terminal Master Key Set";
			MasterKeySet.ForeColor = System.Drawing.Color.DarkGreen;
			ConnectButton.Enabled = true;

			adm.Stop();
		}

		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				t.Abort();
				ctrl.Stop();
				rcv.Stop();

				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.ConnectButton = new System.Windows.Forms.Button();
			this.HostTextBox = new System.Windows.Forms.TextBox();
			this.TerminalTextBox = new System.Windows.Forms.TextBox();
			this.label1 = new System.Windows.Forms.Label();
			this.label2 = new System.Windows.Forms.Label();
			this.DataTextBox = new System.Windows.Forms.TextBox();
			this.MessageTextBox = new System.Windows.Forms.TextBox();
			this.ClearButton = new System.Windows.Forms.Button();
			this.MasterKeySet = new System.Windows.Forms.Label();
			this.SuspendLayout();
			// 
			// ConnectButton
			// 
			this.ConnectButton.Enabled = false;
			this.ConnectButton.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.ConnectButton.Location = new System.Drawing.Point(256, 256);
			this.ConnectButton.Name = "ConnectButton";
			this.ConnectButton.TabIndex = 0;
			this.ConnectButton.Text = "&Send";
			this.ConnectButton.Click += new System.EventHandler(this.ConnectButton_Click);
			// 
			// HostTextBox
			// 
			this.HostTextBox.Location = new System.Drawing.Point(80, 224);
			this.HostTextBox.Name = "HostTextBox";
			this.HostTextBox.Size = new System.Drawing.Size(168, 20);
			this.HostTextBox.TabIndex = 1;
			this.HostTextBox.Text = "localhost";
			// 
			// TerminalTextBox
			// 
			this.TerminalTextBox.Location = new System.Drawing.Point(80, 256);
			this.TerminalTextBox.Name = "TerminalTextBox";
			this.TerminalTextBox.Size = new System.Drawing.Size(168, 20);
			this.TerminalTextBox.TabIndex = 2;
			this.TerminalTextBox.Text = "localhost";
			// 
			// label1
			// 
			this.label1.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.label1.Location = new System.Drawing.Point(16, 224);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(56, 24);
			this.label1.TabIndex = 3;
			this.label1.Text = "Host:";
			this.label1.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// label2
			// 
			this.label2.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.label2.Location = new System.Drawing.Point(16, 256);
			this.label2.Name = "label2";
			this.label2.Size = new System.Drawing.Size(56, 24);
			this.label2.TabIndex = 4;
			this.label2.Text = "Terminal:";
			this.label2.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// DataTextBox
			// 
			this.DataTextBox.AcceptsReturn = true;
			this.DataTextBox.AcceptsTab = true;
			this.DataTextBox.AllowDrop = true;
			this.DataTextBox.Location = new System.Drawing.Point(16, 24);
			this.DataTextBox.Multiline = true;
			this.DataTextBox.Name = "DataTextBox";
			this.DataTextBox.ReadOnly = true;
			this.DataTextBox.ScrollBars = System.Windows.Forms.ScrollBars.Vertical;
			this.DataTextBox.Size = new System.Drawing.Size(312, 72);
			this.DataTextBox.TabIndex = 5;
			this.DataTextBox.Text = "";
			// 
			// MessageTextBox
			// 
			this.MessageTextBox.AcceptsReturn = true;
			this.MessageTextBox.AcceptsTab = true;
			this.MessageTextBox.AllowDrop = true;
			this.MessageTextBox.Location = new System.Drawing.Point(16, 112);
			this.MessageTextBox.Multiline = true;
			this.MessageTextBox.Name = "MessageTextBox";
			this.MessageTextBox.Size = new System.Drawing.Size(312, 72);
			this.MessageTextBox.TabIndex = 6;
			this.MessageTextBox.Text = "";
			// 
			// ClearButton
			// 
			this.ClearButton.Location = new System.Drawing.Point(256, 224);
			this.ClearButton.Name = "ClearButton";
			this.ClearButton.TabIndex = 7;
			this.ClearButton.Text = "&Clear";
			this.ClearButton.Click += new System.EventHandler(this.ClearButton_Click);
			// 
			// MasterKeySet
			// 
			this.MasterKeySet.ForeColor = System.Drawing.Color.Red;
			this.MasterKeySet.Location = new System.Drawing.Point(96, 192);
			this.MasterKeySet.Name = "MasterKeySet";
			this.MasterKeySet.Size = new System.Drawing.Size(152, 16);
			this.MasterKeySet.TabIndex = 8;
			this.MasterKeySet.Text = "Terminal Master Key Not Set";
			// 
			// Terminal
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(346, 293);
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.MasterKeySet,
																		  this.ClearButton,
																		  this.MessageTextBox,
																		  this.DataTextBox,
																		  this.label2,
																		  this.label1,
																		  this.TerminalTextBox,
																		  this.HostTextBox,
																		  this.ConnectButton});
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedSingle;
			this.Name = "Terminal";
			this.Text = "Terminal";
			this.ResumeLayout(false);

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new Terminal());
		}

		private void ConnectButton_Click(object sender, System.EventArgs e) {
			TcpClient host = new TcpClient();
			
			try
			{
				TcpClient terminal = new TcpClient();
				byte[] key = new byte[Host.algorithm.GetKeyLength()];
				byte[] rawip = Host.HostToByteIp(HostTextBox.Text);

				host.Connect(HostTextBox.Text, Host.HOSTPORT);

				NetworkStream strm = host.GetStream();
				strm.Write(rawip, 0, rawip.Length);
				strm.Read(key, 0, key.Length);
				strm.Close();

				host.Close();
						
				while(!sessionKeySet)
				{
					Thread.Sleep(100); //wait for session key
				}

				terminal.Connect(TerminalTextBox.Text, Host.RCVPORT);
				strm = terminal.GetStream();
				byte[] raw = Encoding.ASCII.GetBytes(MessageTextBox.Text);
				raw = facility.EncipherWithKS(raw);
				sessionKeySet = false; //invalidate current session key
				strm.Write(raw, 0, raw.Length);
				MessageTextBox.Clear();
				strm.Close();
				
				terminal.Close();
			}
			catch (SocketException ex)
			{
				System.Windows.Forms.MessageBox.Show(ex.Message);
			}
		}

		private void ClearButton_Click(object sender, System.EventArgs e) {
			DataTextBox.Clear();
			MessageTextBox.Clear();
		}
	}
}
